%% Initial data display and selection of time windows
%% The extracted window is saved in workspace file 'trial_<trial_number>_reduced.mat'
%% together with complete geometry information for the given trial.
%%
%% Typical usage:
%%		first_display();		% in Extract_AC_Signal.m (using 'rec' already in memory)
%%		first_display(103);		% to display and extract reduced data from trial 103
%%		first_display(103,3375);	    % to re-extract data with strike time set to 3375 ms
%%		first_display(103,[],[3300,3500]);  % to change the extraction window 
%%
%% Parameters:
%%	trial_no - trial number of interest. If this parameter is not given,
%%		the data are not loaded from file but obtained from
%%		current workspace (variables 'session' and 'rec'). If we want
%%      we can enter any already executed trial no. (e.g. 009 for Trial 9). 
%%      to get the first display after testing is completed.
%%      We would also need to change the source coords, hammer mass, and
%%      if different from the previously evaluated trial.
%%	strike_time - time of the strike (milliseconds) to set in the output files
%%		In the plot, the times will be shown relative to this time.
%%		If not set in function parameters or empty, strike_time will be determined 
%%		from the maximum amplitude recorded at the receiver closest to the source.
%%	trange - time range to extract (in ms). 
%%		This should be just before (approx. 10ms) the time of
%% 		hammer strike, to the time when the wave amplitude is no longer 
% 		greater than the noise on each accelerometer. This is the useful
%%		record, which will speed up signal processing later. The range
%%		should be separated by tens of milliseconds (e.g. 1.3543, 1.3943;
%%		which is 40 ms different).
%%		If this parameter is not given or is empty,
%%		an interval from -50 ms to 100 ms from 'strike_time' is extracted 
%%	log_filename - name of the log file to use.
%%		If this parameter is not given, and trial_no not given,
%%			log name from 'session' is used;
%%		or if trial_no is given (and session not used), 
%%			then name 'Trial_log.txt' is used
%% Returns:
%%	tstrike_ms - time of the strike in milliseconds, determined from 'strike_time'
%%		or by picking the largest amplitude at the nearest receiver
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function tstrike_ms = first_display( trial_no, strike_time, trange, log_filename )

global sys session rec

%% 1) if asking for trial_no, obtain data and source and receiver parameters for it,
%%	load the appropriate file with raw output from test and the corresponding log file and pattern.
%%	otherwise use current values in 'session'

%% also 2): set accelerometer positions in two rows of the matrix 'src_rec_pos':
%	the first row is the in-line coordinates in cm;
%   	the second row is the cross-line coordinates in cm; 
%	the first column corresponds to the source, and the rest to the receivers,

if nargin >= 1	

	cs = session;	% save the current session state

	session.trial_no = trial_no;

	[ session.log_filename, session.data_filename, session.pattern_filename ] = file_names(trial_no);

	if isempty(session.data_filename)	% error
		return
	end

	if nargin >= 4		% use log file given in the arguments

		session.log_filename = log_filename;
	end
 
	[ line_no, ~, ~, ~, session.pattern, ~, ~, src ] = params_ffid(session.log_filename,trial_no);
			% the returned structure 'src' will contain src.skid and src.offset

	session.line_no = line_no(1);	% the returned value is an array but we need only the first element

	rec = load(session.data_filename,'-ascii');        % this loads the trial data file
		
		% recall, this is a plain text file with 19 columns and 512000 rows
    		% load() simply loads this file as a single matrix 'rec', not a structure

	patt = get_pattern();		% get pattern number session.pattern

	src_rec_pos = [ [ src.skid + patt(1,2); src.offset + patt(1,3) ], transpose(patt(:,2:3)) ]; 
			% source coordinates determined from (src.skid, src.offset) and coordinates of first receiver

	hammer_mass = src.size;

else
	patt = get_pattern();

	src_rec_pos = [ transpose(session.hammer_position_cm*0.01), transpose(patt(:,2:3))];  
				% first column contains the source coordinates (inline then crossline), 
				% and second column (to the end) contains the sensor positions.

	hammer_mass = session.hammer_mass;
end

%src_rec_pos

% determine the distances from the source (first column) to each receiver (the rest): 

src_rec_dist = sqrt( sum( (src_rec_pos(:,2:end) - src_rec_pos(:,1)).^2 ) );

%% 3) determine the sampling interval 'dt' (times are already in milliseconds in 'rec')

dt = rec(2,1) - rec(1,1);

%% 4) Determine strike_time

if nargin < 2 || isempty(strike_time)

	% locate the largest peak in the nearest receiver
	% and not the largest peak in the hammer record; 
    % in the hammer records, there are sometimes weird electrical spikes
    % which could cause a false maximum value

	[ ~, ind ] = min(src_rec_dist);		% index of the nearest receiver

	[~, ind ] = max(abs(rec(:,ind+2)));	% time index of the peak

	strike_time = rec(ind,1);
end

tstrike_ms = strike_time;		% output value

%% 5) obtain time range for output and plot

if nargin < 3 || isempty(trange)	% trange not given in function call; use -50 to +100 ms from strike_time
    
	trange = [ -50.0, 100.0 ] + strike_time;
end

%% 6) select the time window in which the strike occurred 

% which is everything greater than the lower bound of the time range and
% less than the upper bound of the time range)

ind = find(rec(:,1) > trange(1) & rec(:,1) < trange(2));

% this keeps only the selected record

rec = rec(ind,:);   

% Shift the time values to equal 0.0 at the strike_time 

rec(:,1) = rec(:,1) - strike_time;

%%		Returned test of a constant-velocity record. 
%%		The same test is used in accel_dispersion() - I.M.

%% Optional test: replace the records of all receivers with the first one
%% shifted in time proportionally to distance.
%% If you uncomment the following line, you should see records 
%% which are copies of the one in rec(:,3) 
%% progressively shifted away from the source at the constant velocity given by the
%% third parameter in this call: 

%  rec = test_moveout(rec,src_rec_dist,0.05);

%% 7) plot all 15 records in one figure

%  scale every record by peak value
%  plot curves  with vertical offsets, so that the four channels are not
%  overlapped, but aligned by time

gain = 0.6;   % additional gain for plot; this is adjusted to increase
              % vertical spacing between plots such that the plots do not 
              % collide (0.6 was typically used for S7)

figure(1)
clf		  % this clears the figure in case it already exists

ylim_plot = [-1, size(rec,2)-1 ];

% plot distances

subplot(1,5,1)

plot(src_rec_dist,1:length(src_rec_dist),'b-')

ylim( ylim_plot )
set(gca,"Xtick", 0:0.2:1.6 )		% label offsets  sparsely
set(gca,"Ytick",1:(size(rec,2)-2))	% label every receiver

ylabel("Channel")
xlabel('Distance (m)')

% plot the records

subplot(1,5,2:5)

hold on		  % this makes the different plot() commands to overlay the curves

plot(rec(:,1),rec(:,2)/max(abs(rec(:,2)))*gain,'r-')		% hammer record

for i=3:size(rec,2)
  plot(rec(:,1),rec(:,i)/max(abs(rec(:,i)))*gain+i-2,'b-')	% accelerometer records
end

	% plot the range of phase velocities by green lines

phase_vel = [ 0.05, 1.0 ]; 
ind = 1:length(src_rec_dist);

for i=1:length(phase_vel)
	x = src_rec_dist/phase_vel(i);
	plot(x,ind,'g-')
	text(x(end),ind(end)+0.5,[ num2str(phase_vel(i)) 'm/ms'],...
			'color','green','horizontalalignment','center')		% label the velocities 
end

xlim(trange - strike_time )
ylim( ylim_plot )

set(gca,"Ytick",0:1:(size(rec,2)-2))		% label source as 0 and every receiver

xlabel("Time (ms)")
ylabel("Channel number / Acceleration")

title( sprintf("Line %d, Trial %d; %s - %d load cycles; Hammer %1.0f g; Strike time %1.2f ms; Pattern %d", ...
		session.line_no, session.trial_no, session.location, session.dq_load_cycles, ...
		session.hammer_mass, strike_time, session.pattern ) )	
						% added info in plot title - I.M.

legend('Hammer', 'Accel')	

%legend('Hammer', 'Accel_1', 'Accel_2', 'Accel_3', 'Accel_4', 'Accel_5',...
%       'Accel_6', 'Accel_7', 'Accel_8', 'Accel_9', 'Accel_10', 'Accel_11',...
%       'Accel_12', 'Accel_13', 'Accel_14', 'Accel_15')

% restore the state of the current session

if nargin > 0
	session = cs;	
end

end	% end big function

