% Obtains parameters of a given or the last FFID in the observer's log.
% ======================================================================
% The log is scanned for the matching or last FFID number, and contents of the
% matching line are returned.
%
% Parameters:
%	log_file - log file name to use. If empty value [] is given, the standard name
%		obtained by file_names(ffid) is used
%	ffid - FFID number to extract from the log file
%		if ffid=[] (default), parameters of teh last ffid are returned.
%		If the trial number is not found in the log, error state is caused
%		and the program stopped
% Returns:
%	line_par - row with seven numbers (see params_header.m):
%		1 - Line
%		2 - BaseSTN
%		3 - BaseX
%		4 - Station spacing
%		5 - Line spacing
%		6 - Source offset
%		7 - Source skid
%	src_no - source point number
%	src_stn - source surface station number
%	rec_stn - array (row) of two values containing receiver station numbers for the first and last receiver
%	pattern - number of pattern (value of zero means no known pattern)
%	trigger - number of trigger
%	ffid_log - FFID value from the log (in case the requested ffid = [])
%	src - structure with additional source parameters: src.offset, src.skid, src.size (for hammer size, for example) 

function [ line_par, src_no, src_stn, rec_stn, pattern, trigger, ffid_log, src ] = params_ffid(log_file,ffid)

	if nargin < 2
		ffid = [];
	end

	if isempty(log_file)
		log_file = file_names(ffid);
	end

	fid = fopen(log_file,'r');

	if fid < 0
		error("Cannot open file '%s' for reading",log_file)
	end

	line_par = zeros(1,7);	% default in case no line number in the header

	rec_shift = 0;
	src_shift = 0;

	pattern = 0;
	trigger = 0;

	src.offset = 0;
	src.skid = 0;
	src.size = 0;

	s = fgets(fid);		% read first line

	while s > 0

		s = strtrim(s);		% trim spaces

		if length(s) < 1		% skip blank line
			s = fgets(fid);			
			continue
		end

		if s(1) == '%' || s(1) == '#'		% skip commentary but extract header parameters

			[ ~, s ] = strtok(s);		

			[ ind, tok ] = params_header(s);	% try recognizing a keyword

			switch ind
				case 0		% ignore

				case 6		% source offset

					n = str2num(tok);

					if isempty(n)
						src.offset = 0;
					else
						src.offset = n;
					end

				case 7		% source skid

					n = str2num(tok);

					if isempty(n)
						src.skid = 0;
					else
						src.skid = n;
					end

				case 11		% source size

					n = str2num(tok);

					if isempty(n)
						src.size = 0;
					else
						src.size = n;
					end

				case 8		% receiver shift

					n = str2num(tok);

					if isempty(n)
						rec_shift = 0;
					else
						rec_shift = n;
					end

				case 9		% source shift

					n = str2num(tok);

					if isempty(n)
						src_shift = 0;
					else
						src_shift = n;
					end

				case 10		% pattern number

					n = str2num(tok);

					if isempty(n)
						pattern = 0;
					else
						pattern = n;
					end

				otherwise	% output in linepar

					n = str2num(tok);

					if ~isempty(n)
						line_par(1,ind) = n;
					end
			end

			s = fgets(fid);			
			continue
		end

		str = s;				% current line

		[ tok, s ] = strtok(s);			% first token

		src_no = str2num(tok);

		[ tok, s ] = strtok(s);			% second token

		ffid_log = str2num(tok);

		% obtain other parameters

		[ tok, s ] = strtok(s);	

%ffid
%src_shift
%rec_shift
		src_stn = str2num(tok) + src_shift;

		[ tok, s ] = strtok(s);	

		rec_stn(1) = str2num(tok) + rec_shift;

		[ tok, s ] = strtok(s);	

		rec_stn(2) = str2num(tok) + rec_shift;

%rec_stn

		[ tok, s ] = strtok(s);

		trigger = str2num(tok);

		if ffid_log == ffid		% ffid found - make outputs

			fclose(fid);
			return			% done
		end

		s = fgets(fid);		% next line
	end
	
	fclose(fid);

	% if got here, the requested ffid was not found in this log

	if ~isempty(ffid)
		error("No FFID = %d in log file %s",ffid, log_file)
	end
end

